import json
import os
import uuid
import zipfile
import tkinter as tk
from tkinter import ttk, filedialog, simpledialog, messagebox

# --- Constants ---
MANIFEST_FILE_NAME = "store_manifest.json"
APKS_DIR_NAME = "apks"
IMAGES_DIR_NAME = "images"

# --- Helper Class for App Data ---
class AppEntry:
    def __init__(self, apk_path):
        self.apk_path = apk_path
        self.apk_file_name = os.path.basename(apk_path)
        # Initial app name from filename (user can't edit this in this simple GUI)
        self.app_name = os.path.splitext(self.apk_file_name)[0]
        # Editable fields
        self.package_name_var = tk.StringVar(value=self._generate_package_name())
        self.description_var = tk.StringVar(value=f"תיאור עבור {self.app_name}")
        self.image_file_name_var = tk.StringVar(value="") # Path to a user-selected image for this APK

    def _generate_package_name(self):
        safe_app_name = ''.join(e for e in self.app_name.lower() if e.isalnum() or e == '_').replace(" ", "_")
        if not safe_app_name:
            safe_app_name = "unknownapp"
        return f"com.generated.{safe_app_name}.{uuid.uuid4().hex[:6]}"

    def to_dict(self):
        return {
            "name": self.app_name,
            "description": self.description_var.get(),
            "apkFileName": self.apk_file_name,
            "imageFileName": os.path.basename(self.image_file_name_var.get()) if self.image_file_name_var.get() else None,
            "packageName": self.package_name_var.get()
        }

# --- Main Application Class ---
class BackupCreatorApp:
    def __init__(self, root_window):
        self.root = root_window
        self.root.title("Kosher App Store - Backup Creator")
        self.root.geometry("750x750") # Adjusted size for more fields

        self.selected_apks_data = [] # List of AppEntry objects

        # --- Main PanedWindow for layout ---
        paned_window = ttk.PanedWindow(root_window, orient=tk.HORIZONTAL)
        paned_window.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)

        # --- Left Pane: Settings and APK List ---
        left_pane = ttk.Frame(paned_window, padding="10")
        paned_window.add(left_pane, weight=1)

        # Store Settings Frame
        settings_frame = ttk.LabelFrame(left_pane, text="Store General Settings", padding="10")
        settings_frame.pack(pady=10, fill="x")
        self._create_settings_widgets(settings_frame)

        # APKs List Frame
        apks_list_frame = ttk.LabelFrame(left_pane, text="APKs to Include", padding="10")
        apks_list_frame.pack(pady=10, fill="both", expand=True)
        self._create_apk_list_widgets(apks_list_frame)

        # --- Right Pane: Selected APK Details ---
        right_pane = ttk.Frame(paned_window, padding="10")
        paned_window.add(right_pane, weight=2)

        self.selected_apk_details_frame = ttk.LabelFrame(right_pane, text="Edit Selected APK Details", padding="10")
        self.selected_apk_details_frame.pack(fill="both", expand=True)
        self._create_apk_details_widgets(self.selected_apk_details_frame)

        # --- Action Buttons Frame ---
        action_buttons_frame = ttk.Frame(root_window, padding="10")
        action_buttons_frame.pack(fill="x")
        self._create_action_buttons(action_buttons_frame)

        self.clear_apk_details_fields() # Initialize details fields as empty/disabled

    def _create_settings_widgets(self, parent_frame):
        row_num = 0
        ttk.Label(parent_frame, text="Store Password:").grid(row=row_num, column=0, sticky="w", padx=5, pady=2)
        self.password_var = tk.StringVar(value="yossi6547")
        ttk.Entry(parent_frame, textvariable=self.password_var, width=30).grid(row=row_num, column=1, sticky="ew", padx=5, pady=2)
        row_num += 1

        self.normal_install_var = tk.BooleanVar(value=False)
        ttk.Checkbutton(parent_frame, text="Default: Normal Install Mode", variable=self.normal_install_var).grid(row=row_num, column=0, columnspan=2, sticky="w", padx=5, pady=2)
        row_num += 1

        self.use_root_var = tk.BooleanVar(value=True)
        ttk.Checkbutton(parent_frame, text="Default: Use Root Install", variable=self.use_root_var).grid(row=row_num, column=0, columnspan=2, sticky="w", padx=5, pady=2)
        row_num += 1

        ttk.Label(parent_frame, text="Default Root Method:").grid(row=row_num, column=0, sticky="w", padx=5, pady=2)
        self.root_method_var = tk.StringVar(value="pm_install")
        pm_radio = ttk.Radiobutton(parent_frame, text="PM Install", variable=self.root_method_var, value="pm_install")
        pm_radio.grid(row=row_num, column=1, sticky="w", padx=5, pady=0)
        row_num += 1
        copy_radio = ttk.Radiobutton(parent_frame, text="Copy to System", variable=self.root_method_var, value="copy_to_system")
        copy_radio.grid(row=row_num, column=1, sticky="w", padx=5, pady=0)
        row_num += 1

        ttk.Label(parent_frame, text="Default Copy Location:").grid(row=row_num, column=0, sticky="w", padx=5, pady=2)
        self.install_location_var = tk.StringVar(value="system/app")
        sys_app_radio = ttk.Radiobutton(parent_frame, text="/system/app", variable=self.install_location_var, value="system/app")
        sys_app_radio.grid(row=row_num, column=1, sticky="w", padx=5, pady=0)
        row_num += 1
        priv_app_radio = ttk.Radiobutton(parent_frame, text="/system/priv-app", variable=self.install_location_var, value="system/priv-app")
        priv_app_radio.grid(row=row_num, column=1, sticky="w", padx=5, pady=0)
        row_num += 1

        self.send_feedback_var = tk.BooleanVar(value=True)
        ttk.Checkbutton(parent_frame, text="Enable Send Feedback in Menu", variable=self.send_feedback_var).grid(row=row_num, column=0, columnspan=2, sticky="w", padx=5, pady=2)
        row_num += 1

        ttk.Label(parent_frame, text="About Text:").grid(row=row_num, column=0, sticky="nw", padx=5, pady=2)
        self.about_text_widget = tk.Text(parent_frame, height=4, width=30, wrap=tk.WORD)
        self.about_text_widget.grid(row=row_num, column=1, sticky="ew", padx=5, pady=2)
        self.about_text_widget.insert(tk.END, "זוהי חנות אפליקציות כשרה שנוצרה באמצעות סקריפט הגיבוי.")
        
        parent_frame.columnconfigure(1, weight=1)

    def _create_apk_list_widgets(self, parent_frame):
        self.apk_listbox = tk.Listbox(parent_frame, selectmode=tk.SINGLE, height=10)
        self.apk_listbox.pack(side=tk.LEFT, fill="both", expand=True, padx=(0, 5))
        self.apk_listbox.bind("<<ListboxSelect>>", self.on_apk_select_from_list)

        apk_scrollbar = ttk.Scrollbar(parent_frame, orient="vertical", command=self.apk_listbox.yview)
        apk_scrollbar.pack(side=tk.LEFT, fill="y")
        self.apk_listbox.config(yscrollcommand=apk_scrollbar.set)

        buttons_subframe = ttk.Frame(parent_frame)
        buttons_subframe.pack(side=tk.BOTTOM, fill="x", pady=(5,0))
        add_apks_button = ttk.Button(buttons_subframe, text="Add APK(s)", command=self.add_apks_handler)
        add_apks_button.pack(side=tk.LEFT, padx=2)
        remove_apk_button = ttk.Button(buttons_subframe, text="Remove Selected", command=self.remove_selected_apk_handler)
        remove_apk_button.pack(side=tk.LEFT, padx=2)


    def _create_apk_details_widgets(self, parent_frame):
        self.selected_apk_filename_label = ttk.Label(parent_frame, text="APK File: N/A")
        self.selected_apk_filename_label.pack(anchor="w", pady=2)

        self.selected_apk_appname_label = ttk.Label(parent_frame, text="App Name (from filename): N/A")
        self.selected_apk_appname_label.pack(anchor="w", pady=2)

        ttk.Label(parent_frame, text="Package Name (auto-generated, editable):").pack(anchor="w", pady=(10,0))
        self.selected_apk_package_entry = ttk.Entry(parent_frame, width=40)
        self.selected_apk_package_entry.pack(anchor="w", fill="x", pady=2)
        self.selected_apk_package_entry.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)

        ttk.Label(parent_frame, text="Description:").pack(anchor="w", pady=(10,0))
        self.selected_apk_description_text = tk.Text(parent_frame, height=5, width=40, wrap=tk.WORD)
        self.selected_apk_description_text.pack(anchor="w", fill="x", pady=2)
        self.selected_apk_description_text.bind("<KeyRelease>", self.update_selected_apk_details_from_fields)
        
        # Placeholder for image selection - can be expanded
        self.selected_apk_image_label_var = tk.StringVar(value="Image: None")
        ttk.Label(parent_frame, textvariable=self.selected_apk_image_label_var).pack(anchor="w", pady=(10,0))
        # ttk.Button(parent_frame, text="Select Image for this APK", command=self.select_image_for_apk).pack(anchor="w", pady=2)


    def _create_action_buttons(self, parent_frame):
        create_button = ttk.Button(parent_frame, text="Create Backup ZIP", command=self.create_backup_zip_handler, style="Accent.TButton")
        create_button.pack(pady=10)
        style = ttk.Style()
        style.configure("Accent.TButton", font=("Segoe UI", 10, "bold"))


    def add_apks_handler(self):
        files = filedialog.askopenfilenames(
            title="Select APK files",
            filetypes=(("APK files", "*.apk"), ("All files", "*.*"))
        )
        if files:
            for f_path in files:
                if not any(entry.apk_path == f_path for entry in self.selected_apks_data):
                    app_entry = AppEntry(f_path)
                    self.selected_apks_data.append(app_entry)
                    self.apk_listbox.insert(tk.END, app_entry.apk_file_name)
            if self.selected_apks_data and self.apk_listbox.size() > 0:
                 self.apk_listbox.selection_clear(0, tk.END)
                 self.apk_listbox.selection_set(0)
                 self.on_apk_select_from_list(None)

    def remove_selected_apk_handler(self):
        selected_indices = self.apk_listbox.curselection()
        if selected_indices:
            index_to_remove = selected_indices[0]
            self.apk_listbox.delete(index_to_remove)
            del self.selected_apks_data[index_to_remove]
            
            if self.apk_listbox.size() > 0:
                new_selection_index = max(0, index_to_remove - 1)
                if new_selection_index < self.apk_listbox.size():
                    self.apk_listbox.selection_set(new_selection_index)
                    self.on_apk_select_from_list(None)
                else: # if last item was removed
                    self.clear_apk_details_fields()
            else: # List is now empty
                self.clear_apk_details_fields()


    def on_apk_select_from_list(self, event):
        selected_indices = self.apk_listbox.curselection()
        if selected_indices:
            index = selected_indices[0]
            app_entry = self.selected_apks_data[index]
            
            self.selected_apk_filename_label.config(text=f"APK File: {app_entry.apk_file_name}")
            self.selected_apk_appname_label.config(text=f"App Name (from filename): {app_entry.app_name}")

            self.selected_apk_package_entry.delete(0, tk.END)
            self.selected_apk_package_entry.insert(0, app_entry.package_name_var.get())

            self.selected_apk_description_text.delete("1.0", tk.END)
            self.selected_apk_description_text.insert("1.0", app_entry.description_var.get())

            # self.selected_apk_image_label_var.set(f"Image: {os.path.basename(app_entry.image_file_name_var.get()) if app_entry.image_file_name_var.get() else 'None'}")

            self.selected_apk_package_entry.config(state=tk.NORMAL)
            self.selected_apk_description_text.config(state=tk.NORMAL)
        else:
            self.clear_apk_details_fields()

    def update_selected_apk_details_from_fields(self, event):
        selected_indices = self.apk_listbox.curselection()
        if selected_indices:
            index = selected_indices[0]
            self.selected_apks_data[index].package_name_var.set(self.selected_apk_package_entry.get())
            self.selected_apks_data[index].description_var.set(self.selected_apk_description_text.get("1.0", tk.END).strip())
    
    # def select_image_for_apk(self):
    # This would be more complex, requiring to know which APK is selected,
    # then opening a file dialog for an image, and storing its path.

    def clear_apk_details_fields(self):
        self.selected_apk_filename_label.config(text="APK File: N/A")
        self.selected_apk_appname_label.config(text="App Name (from filename): N/A")
        self.selected_apk_package_entry.delete(0, tk.END)
        self.selected_apk_package_entry.config(state=tk.DISABLED)
        self.selected_apk_description_text.delete("1.0", tk.END)
        self.selected_apk_description_text.config(state=tk.DISABLED)
        # self.selected_apk_image_label_var.set("Image: None")

    def create_backup_zip_handler(self):
        if not self.selected_apks_data:
            messagebox.showerror("Error", "No APK files selected.")
            return

        output_zip_path = filedialog.asksaveasfilename(
            title="Save Backup ZIP file as",
            defaultextension=".zip",
            filetypes=(("ZIP files", "*.zip"), ("All files", "*.*"))
        )
        if not output_zip_path:
            messagebox.showwarning("Cancelled", "Save location not selected.")
            return

        app_info_list_for_json = [entry.to_dict() for entry in self.selected_apks_data]
        store_config_data = {
            "appList": app_info_list_for_json,
            "settingsPassword": self.password_var.get(),
            "normalInstallMode": self.normal_install_var.get(),
            "useRootInstall": self.use_root_var.get(),
            "rootInstallMethod": self.root_method_var.get(),
            "installLocation": self.install_location_var.get(),
            "enableSendFeedback": self.send_feedback_var.get(),
            "aboutText": self.about_text_widget.get("1.0", tk.END).strip()
        }
        manifest_json_content = json.dumps(store_config_data, indent=4, ensure_ascii=False)

        try:
            with zipfile.ZipFile(output_zip_path, 'w', zipfile.ZIP_DEFLATED) as zf:
                zf.writestr(MANIFEST_FILE_NAME, manifest_json_content)
                for app_entry in self.selected_apks_data:
                    zip_apk_path = os.path.join(APKS_DIR_NAME, app_entry.apk_file_name)
                    zf.write(app_entry.apk_path, zip_apk_path)
                    # Image handling would go here if implemented
            messagebox.showinfo("Success", f"Backup created successfully at:\n{output_zip_path}")
        except Exception as e:
            messagebox.showerror("Error", f"Failed to create backup: {e}")

if __name__ == "__main__":
    root = tk.Tk()
    app = BackupCreatorApp(root)
    root.mainloop()